#ifndef __KYSDK_MODULE_AUTHORIZE_H__
#define __KYSDK_MODULE_AUTHORIZE_H__

#include <bits/types.h>
#include <linux/limits.h>

#define KDK_ERROR_OPERATOR_FAILED          (-1)
#define KDK_ERROR_INVALID_PARAMETER        (-2)
#define KDK_ERROR_GET_PROGRAM_PATH         (-3)
#define KDK_ERROR_GET_CONF_FAILED          (-4)
#define KDK_ERROR_SET_CONF_FAILED          (-5)
#define KDK_ERROR_ADD_CONF_KEY_FAILED      (-6)
#define KDK_ERROR_DEL_CONF_KEY_FAILED      (-7)
#define KDK_ERROR_FREE_FAILED              (-8)
#define KDK_ERROR_OUT_OF_MEMORY            (-9)
#define KDK_ERROR_DIRTY_MEMORY             (-10)

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    KDK_PERMISSION_APPINSTSEC = 0,     /// 应用安装安全（黑白名单）
    KDK_PERMISSION_APPEXECTLSEC,       /// 应用执行控制安全（pblk）
    KDK_PERMISSION_APPUNINSTSEC,       /// 应用防卸载安全（防卸载）
    KDK_PERMISSION_DEVICESEC,          /// 设备安全 (devctl)
    KDK_PERMISSION_FILESEC,            /// 文件安全 (fpro)
    KDK_PERMISSION_KMODSEC,            /// 内核模块安全 (kmodpro)
    KDK_PERMISSION_PROCESSSEC,         /// 进程安全 (pblk+pro)
    KDK_PERMISSION_NETSEC,             /// 网络安全
    KDK_PERMISSION_MAX_NUM,
} kdk_permission_type_e;

static const char * const kdk_module_array[KDK_PERMISSION_MAX_NUM] = {
    "appinstsec",
    "appexectlsec",
    "appuninstsec",
    "devicesec",
    "filesec",
    "kmodsec",
    "processsec",
    "netsec",
};

typedef struct {
    char program_path[PATH_MAX];
    void * pnext;
}kdk_program_list_t;

typedef struct {
    char module_name[PATH_MAX];
    void * pnext;
}kdk_module_info_t;

typedef struct {
    char program_path[PATH_MAX];
    kdk_module_info_t * module_list;
    void * pnext;
}kdk_program_info_t;

typedef struct {
    char package[PATH_MAX];
    kdk_program_info_t * program_list;
    void * pnext;
}kdk_package_info_t;

/*
查询所有第三方软件包信息
* 输入参数：
*  无
* 输出参数：
*  package_info：。第三方套件授权信息，package_info指针的指向的内存由
*  函数内部申请，在调用方使用完list指针后，需要调用方主动调用free_package_info函数释放内存。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_get_all_third_party_package_info(kdk_package_info_t** package_info);

/*
增加指定package的指定进程的指定类型的安全接口授权
* 输入参数：
*  package：指定的软件包名
*  program_path：指定可执行文件路径
*  type：指定的安全接口类型，参考kdk_permission_type_e
* 输出参数：
* 无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_add_permission(const char * package, const char * program_path, kdk_permission_type_e type);

/*
删除指定package的指定进程的指定类型的安全接口授权
* 输入参数：
*  package：指定的软件包名
*  program_path：指定可执行文件路径
*  type：指定的安全接口类型，参考kdk_permission_type_e
* 输出参数：
*  无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_del_permission(const char * package,const char * program_path,kdk_permission_type_e type);

/*
删除指定package的安全模块接口授权
* 输入参数：
*  package：指定的软件包名
* 输出参数：
*  无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_del_package(const char * package);

/*
删除指定可执行文件的安全模块接口授权
* 输入参数：
*  package：指定的软件包名
* 输出参数：
*  无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_del_program(const char * package,const char * program_path);

/*
查询指定的pid的进程是否具备某类安全接口操作权限
* 输入参数：
*  pid：指定程序的pid
*  type：指定的安全接口类型,参考kdk_permission_type_e
* 输出参数：
*  permission：指定程序授权状态，0-指定程序没有被授权，1-指定程序已经被授权。
*  package : 软件包名，长度是PATH_MAX，如果不需要使用此参数，可传入NULL

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_get_permission_bypid(pid_t pid, kdk_permission_type_e type, int* permission, char* package);

/*
检查指定安全接口类型是否可用（设备管控等独占权限，如果被涉密安全套件占用，则此接口类型不可用）
* 输入参数：
*  type：指定的安全接口类型,参考kdk_permission_type_e
* 输出参数：
*  无

* 返回值：
*  0：  不可用
*  1：  可用
*/
int kdk_check_permission_enable(kdk_permission_type_e type);

/*
释放package_info接口
* 输入参数：
*  package_info：package_info指针
* 输出参数：
*  无
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_free_package_info(kdk_package_info_t *package_info);

/*
检查当前是否已经授权某种安全权限（主要检查设备管控等独占权限是否已被授权）
* 输入参数：
*  type： 安全权限类型
* 输出参数：
*  无

* 返回值：
*  0：  当前没有授权此安全权限
*  1：  当前已经授权此安全权限
*/
int kdk_check_has_permission(kdk_permission_type_e type);

/*
增加指定package的指定进程的指定类型的安全接口授权拓展函数
* 输入参数：
*  package：指定的软件包名
*  program_path：指定可执行文件路径
*  module_type：指定的安全接口类型名称
*  third_party:第三方安全套件标识， 0-指定软件包不是第三方安全套件，指定软件包是第三方套件
*  enable:授权开关标志
* 输出参数：
* 无。

* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_add_permission_ext(const char * package,const char * program_path,int third_party,int enable,const char *module_name);

/*
按条件查询软件包授权信息
* 输入参数：
*  conf_path：授权文件路径
*  third_party：第三方软件标志、查询条件
*  enable：  授权开关标志、查询条件
* 输出参数：
*  package_info：。软件包授权信息，package_info指针的指向的内存由
*  函数内部申请，在调用方使用完package_info指针后，需要调用方主动调用kdk_free_package_info函数释放内存。
* 返回值：
*  0：  成功
*  其他：由错误码定义的错误类型
*/
int kdk_get_package_info_by_path(const char *conf_path,int third_party,int enable,kdk_package_info_t** package_info);

#ifdef __cplusplus
}
#endif

#endif
